/*
 *   Gnome Nibbles: Gnome Worm Game
 *   Written by Sean MacIsaac <sjm@acm.org>, Ian Peters <itp@gnu.org>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _SCOREBOARD_H_
#define _SCOREBOARD_H_

#include <gtk/gtk.h>

#include "gnibbles.h"
#include "worm.h"

typedef struct {
  GnibblesWorm *worms[NUMWORMS];
  GtkWidget *names[NUMWORMS];
  GtkWidget *data[NUMWORMS];
  gint count;
} GnibblesScoreboard;

GnibblesScoreboard *gnibbles_scoreboard_new (GtkWidget * t_statusbar);

void gnibbles_scoreboard_register (GnibblesScoreboard * scoreboard,
                                   GnibblesWorm * t_worm,
                                   gchar * colorname);

void gnibbles_scoreboard_update (GnibblesScoreboard * scoreboard);

void gnibbles_scoreboard_clear (GnibblesScoreboard * scoreboard);

#endif
