/****************************************************************************
 *                                                                          *
 *                      Velena Source Code V1.0                             *
 *                   Written by Giuliano Bertoletti                         *
 *       Based on the knowledged approach of Louis Victor Allis             *
 *   Copyright (C) 1996-97 by Giuliano Bertoletti & GBE 32241 Software PR   *
 *                                                                          *
 ****************************************************************************

 Portable engine version.
 read the README file for further informations.

 ===========================================================================*/


#define CLAIMEVEN       1
#define BASEINVERSE     2
#define VERTICAL        3
#define AFTEREVEN       4
#define LOWINVERSE      5
#define HIGHINVERSE     6
#define BASECLAIM       7
#define BEFORE          8
#define SPECIALBEFORE   9
